#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "pngio.h"


void usage(char *pname)
{
  exit(fprintf(stderr,
"usage: %s <options>\n"
"  options are:\n"
"\n"
"   -r             reverse operation convert .png to .rle\n"
"   -i inputfile\n"
"   -o outputfile\n"
"   -w width\n"
"   -h height\n"
"\n"
"  width and should be specified only when converting from\n"
"  .rle to .png\n",pname));
}

int shift(int *argc,char **argv, int i,int a)
{
  for(;i+a<*argc;i++)
    argv[i]= argv[i+a];
  argv[i]= NULL;
  *argc -= a;
}

char *get_arg(int *argc,char **argv,int *i)
{
  char *R;
  if (*i+1>=*argc) usage(argv[0]);

  R= argv[*i+1];
  shift(argc,argv,*i,2);
  *i-= 1;
  return R;
}

int write_png(char *outfn, unsigned int *buf, int w,int h)
{
  char *e;
  e= write_png_image(outfn,w,h,buf);
  if (e)
   exit(fprintf(stderr,"error writing png image %s: %s\n",outfn, e));
  free(buf);
  return 0;
}

int encode_png(char *infn, char *outfn, int width, int height)
{
  FILE *f;
  unsigned char *raw_buffer;
  unsigned int  *proper_buffer;
  unsigned int i,sz,R;
  fprintf(stderr,"decoding from %s (%d x %d) to %s \n", infn, width, height, outfn);
  f= fopen(infn, "rb");
  if (!f) exit(fprintf(stderr,"can't open %s: %s\n", infn, strerror(errno)));
  sz= width*height*4;
  raw_buffer= malloc(sz);
  R= fread(raw_buffer, 1, sz, f);
  if (R!=sz)
     exit(fprintf(stderr,"read returned a short count. expected %d, "
                         " returned %d\n", sz, R));
  fclose(f);
  proper_buffer= malloc(width*height*sizeof(unsigned int)); 
  for(i=0;i<sz/4;i++)
  {
    proper_buffer[i]= 0xff000000 |
                raw_buffer[4*i] |
                (raw_buffer[4*i+1] << 8) |
                (raw_buffer[4*i+2] << 16);
  }
  free(raw_buffer);
  return write_png(outfn,proper_buffer,width, height);
}

int encode_rle(char *infn, char *outfn)
{
  FILE *f;
  unsigned int width, height, *proper_buffer, i, sz;
  unsigned char *raw_buffer;
  int R;
  char *e;
  fprintf(stderr,"encoding from %s to %s \n", infn, outfn);
  f= fopen(infn, "rb");
  e= read_png_image(infn, &width, &height, &proper_buffer);
  if (e)
     exit(fprintf(stderr,"can't read png image %s: %s\n", infn, e));
  fprintf(stderr,"image is %d x %d\n", width,height);

  sz= width*height;
  raw_buffer= malloc(sz*4);
  for(i=0;i<sz;i++)
  {
    raw_buffer[4*i]= proper_buffer[i];
    raw_buffer[4*i+1]= proper_buffer[i] >> 8;
    raw_buffer[4*i+2]= proper_buffer[i] >> 16;
    raw_buffer[4*i+3]= 0xff;
  }
  free(proper_buffer);

  f= fopen(outfn,"wb");
  if (!f) exit(fprintf(stderr,"can't open %s: %s\n",outfn,strerror(errno)));
  R= fwrite(raw_buffer, 1, sz*4, f);
  if (R!=sz*4)
      exit(fprintf(stderr,"write(%s) returned short count %d, "
                     "was expecting %d\n", outfn, R, sz*4));

  fclose(f);
  free(raw_buffer);
  return 0;
}
int main(int argc,char **argv)
{
  int i;
  int reverse= 0;
  char *outf= NULL, *inf= NULL;
  int width=-800, height= -480;

  for(i=1;i<argc;i++)
   if (!strcmp(argv[i],"-w"))
     width= atoi(get_arg(&argc,argv,&i));
   else if (!strcmp(argv[i],"-h"))
     height= atoi(get_arg(&argc,argv,&i));
   else if (!strcmp(argv[i],"-o"))
     outf= get_arg(&argc,argv,&i);
   else if (!strcmp(argv[i],"-r"))
     { reverse= 1; shift(&argc,argv,i,1); i--; }
   else if (!strcmp(argv[i],"-i"))
     inf= get_arg(&argc,argv,&i);
   else usage(argv[0]);

   if (!inf)
      inf= reverse ? "input.png" : "input.rle";

   if (!outf)
      outf= reverse ? "output.rle" : "output.png";

   if (reverse && (width>=0 || height>=0))
      usage(argv[0]);

   if (reverse)
      return encode_rle(inf, outf);

   if (width<0) width= -width;
   if (height<0) height= -height;

   return encode_png(inf, outf,width,height);
}
